<?php

function seosara_wp_wpd_add_saraID_filter_link($views) {
    $saraID_filter_link = admin_url('edit.php?post_type=page&saraID_filter=1');
    $current_filter = isset($_GET['saraID_filter']) ? $_GET['saraID_filter'] : '';

    // Query to get the count of posts with the 'saraID' custom meta key
    $pages_with_saraID = new WP_Query([
        'post_type'  => 'page',
        'meta_query' => [
            [
                'key'     => 'saraID',
                'compare' => 'EXISTS',
            ],
        ],
        'fields'         => 'ids', // Only get post IDs to speed up the query
        'posts_per_page' => -1,    // Retrieve all posts that match the query
        'no_found_rows'  => true,  // Skip pagination, improve performance for counting
    ]);

    // Count the results
    $count = $pages_with_saraID->post_count;

    // Add a custom class to highlight when the filter is active
    $class = $current_filter == '1' ? 'current' : '';

    // Construct the link with the count
    $views['saraID_filter'] = "<a href='{$saraID_filter_link}' class='{$class}'>SEOSARA <span class='count'>($count)</span></a>";

    return $views;
}

function seosara_wp_wpd_add_saraID_filter_link_post($views) {
    // Construct the filter link for the admin
    $saraID_filter_link = admin_url('edit.php?post_type=post&saraID_filter=1');
    $current_filter = isset($_GET['saraID_filter']) ? $_GET['saraID_filter'] : '';

    // Query to get the count of posts with the 'saraID' custom meta key
    $pages_with_saraID = new WP_Query([
        'post_type'  => 'post',
        'meta_query' => [
            [
                'key'     => 'saraID',
                'compare' => 'EXISTS',
            ],
        ],
        'fields'         => 'ids', // Only get post IDs to speed up the query
        'posts_per_page' => -1,    // Retrieve all posts that match the query
        'no_found_rows'  => true,  // Skip pagination, improve performance for counting
    ]);

    // Get the total count of matching posts
    $count = $pages_with_saraID->post_count;
    
    // Add a custom class to highlight when the filter is active
    $class = $current_filter == '1' ? 'current' : '';

    // Construct the link with the count
    $views['saraID_filter'] = "<a href='{$saraID_filter_link}' class='{$class}'>SEOSARA <span class='count'>($count)</span></a>";

    return $views;
}

function seosara_wp_seosara_add_settings_page() {
    // Add the main menu item with a callback that performs a redirect
	add_menu_page(__('SEOSARA.AI', 'menu-test'), __('SEOSARA', 'menu-test'), 'manage_options', 'seosara-main-redirect', 'seosara_wp_main_menu_redirect', 'dashicons-admin-site', 6);

    // Add a new submenu item for the custom redirect
    add_submenu_page('seosara-main-redirect', __('SEOSARA Posts', 'menu-test'), __('SEOSARA Posts', 'menu-test'), 'manage_options', 'seosara-filtered-posts', 'seosara_wp_filtered_posts_redirect');

    // Add the first submenu item with the original functionality
    add_submenu_page('seosara-main-redirect', __('Order Content', 'menu-test'), __('Order Content', 'menu-test'), 'manage_options', 'seosara-order-content', 'seosara_wp_order_content_page');

    // Add the 'API Key' submenu page
    add_submenu_page('seosara-main-redirect', __('API Key', 'menu-test'), __('Settings', 'menu-test'), 'manage_options', 'seosara-api-key', 'seosara_wp_api_key_page');
    // Append "Pages" to the automatically created submenu item under the "SEOSARA" main menu
    global $submenu;
    if (isset($submenu['seosara-main-redirect'][0][0])) {
        $submenu['seosara-main-redirect'][0][0] .= ' Pages';
    }
}

function seosara_wp_main_menu_redirect() {
    wp_redirect(admin_url('edit.php?post_type=page&saraID_filter=1'));
    exit;
}

function seosara_wp_filtered_posts_redirect() {
    // Perform the redirect
    wp_redirect(admin_url('edit.php?post_type=post&saraID_filter=1'));
    exit;
}
	 
function seosara_wp_check_current_seosara_screen() {
    if (isset($_GET['post_type']) && ($_GET['post_type'] === 'page' || $_GET['post_type'] === 'post') && isset($_GET['saraID_filter']) && $_GET['saraID_filter'] == '1') {
        // Get necessary options
        $api_key = get_option('seosara_api_key');
        $site_url = get_option('siteurl');
        $admin_email = get_option('seosara_email');

        // Prepare the data to be sent
        $body = [
            'website' => $site_url,
            'admin_email' => $admin_email,
            'key' => $api_key,
        ];
        $body = json_encode($body);

        // Send the POST request
        $response = wp_remote_post('https://bot.seosara.ai/bot-GPT_retrieve.php', [
            'method' => 'POST',
            'body' => $body,
            'headers' => [
                'Authorization' => 'Bearer ' . $api_key,
            ],
            'timeout' => 45,
            'redirection' => 5,
            'httpversion' => '1.0',
            'blocking' => true,
            'sslverify' => true, // Set to false only in a non-production environment
        ]);

        if (is_wp_error($response)) {
            $error_message = $response->get_error_message();
            set_transient('seosara_admin_notice', ['message' => $error_message, 'type' => 'error'], 3600); // 1 hour
            return;
        }

        $body = wp_remote_retrieve_body($response);
        $data = json_decode($body, true);

        if (!is_array($data)) {
            set_transient('seosara_admin_notice', ['message' => 'Invalid API response. ', 'type' => 'error'], 3600); // 1 hour
            return;
        }

        if (isset($data['status']) && $data['status'] === 'error') {
            set_transient('seosara_admin_notice', ['message' => $data['message'], 'type' => 'error'], 3600); // 1 hour
            return;
        }

        if (!isset($data['content'])) {
            set_transient('seosara_admin_notice', ['message' => 'No Content Ready. ', 'type' => 'success'], 3600); // 1 hour
            return;
        }

        foreach ($data['content'] as $content_item) {
            $new_image_urls = [];
            $featured_image_id = null;

            for ($i = 1; $i <= 3; $i++) {
                $image_url = $content_item["img$i"];
                $post_content = $content_item['post'];

                if (!empty($image_url)) {
                    // Extract alt attribute value from the original image tag
                    $alt_pattern = '/<img[^>]*?src=["\']' . preg_quote($image_url, '/') . '["\'][^>]*?alt=["\'](.*?)["\'][^>]*>/i';
                    preg_match($alt_pattern, $post_content, $alt_matches);
                    $alt_text = $alt_matches[1] ?? ''; // Default to empty string if not found

                    $tmp = download_url($image_url);

                    if (is_wp_error($tmp)) {
                        set_transient('seosara_admin_notice', ['message' => 'Error downloading image: ' . $tmp->get_error_message(), 'type' => 'error'], 3600);
                    } else {
                        $parsed_url = parse_url($image_url);
                        $path = $parsed_url['path'] ?? '';
                        $filename = basename($path);
                        $file_array = ['name' => $filename, 'tmp_name' => $tmp];
                        $id = media_handle_sideload($file_array, 0);

                        if (is_wp_error($id)) {
                            set_transient('seosara_admin_notice', ['message' => 'Error uploading image: ' . $id->get_error_message(), 'type' => 'error'], 3600);
                            @unlink($tmp); // Remove the temporary file
                        } else {
                            // Update alt and title for the uploaded image
                            update_post_meta($id, '_wp_attachment_image_alt', $alt_text);
                            wp_update_post([
                                'ID'         => $id,
                                'post_title' => $alt_text, // Set title as alt text
                            ]);

                            $new_image_urls[$image_url] = wp_get_attachment_url($id);

                            if ($i === 1 && !$featured_image_id) {
                                $featured_image_id = $id;
                            }
                        }
                    }
                }
            }

            // Replace old image URLs with new ones in post content and wrap them with an <a> tag
            $post_content = $content_item['post'];
            foreach ($new_image_urls as $old_url => $new_url) {
				    // Escape special characters for regular expression, except for '?' and '#' which can be part of URL query and fragment
				    $escaped_url = preg_quote($old_url, '/');
				    $escaped_url = str_replace(['\?', '\#'], ['?', '#'], $escaped_url); // Unescape '?' and '#'

				    // Pattern to match the <img> tag with the specific src, using non-greedy matching for attributes
				    $pattern = '/(<img\s+[^>]*?src=["\'])' . $escaped_url . '(["\'][^>]*?>)/i';

				    // Replacement pattern reconstructs the <img> tag with the new URL
				    $replacement = '$1' . $new_url . '$2';

				    // Wrap the <img> tag with an <a> tag using the new URL
				    $replacement = '<a href="' . $new_url . '" class="SEOSARAbox">' . $replacement . '</a>';

				    // Perform the replacement in the post content
				    $post_content = preg_replace($pattern, $replacement, $post_content);
            }
          // Generate a meta description from the first 50 words of the post content
	          $words = explode(' ', preg_replace('/\s+/', ' ', wp_strip_all_tags($post_content)), 21); // Split content into words
	          if(count($words) >= 21) {
	            array_pop($words); // Remove the 51st element, which is partially or not needed
     		      $meta_description = trim(implode(' ', $words)) . '...'; // Rejoin the first 50 words and append '...'
	          } else {
     		      $meta_description = trim(implode(' ', $words)); // If less than 50 words, use the whole content   
	          }
				$the_type = get_option('seosara_post_type');
				$the_status = get_option('seosara_post_status');
            // Check for an existing post with the same 'saraID' meta value
            $existing_posts = get_posts([
                'post_type' => $the_type,
                'meta_key' => 'saraID',
                'meta_value' => $content_item['saraID'],
                'numberposts' => 1,
            ]);

            $post_id = 0;
            if ($existing_posts) {
                $post_id = $existing_posts[0]->ID;
                wp_update_post([
                    'ID' => $post_id,
                    'post_title' => wp_strip_all_tags($content_item['title']),
                    'post_content' => $post_content,
                    'post_name' => $content_item['post_name'],
                    'post_excerpt'=> $meta_description,
                ]);
            } else {
                $post_id = wp_insert_post([
                    'post_title' => wp_strip_all_tags($content_item['title']),
                    'post_content' => $post_content,
                    'post_name' => $content_item['post_name'],
                    'post_status' => $the_status,
                    'post_type' => $the_type,
                    'post_author' => $content_item['customerID'],
                    'post_excerpt'=> $meta_description,
                ]);
            }
            if($the_type == 'post') {
	            // Check and create categories
   	         $mainCatId = seosara_wp_ensure_category_exists($content_item['mainCat']);
      	      $subCatId = seosara_wp_ensure_category_exists($content_item['subCat']);
         	   // Set categories for the post
            	if (!is_wp_error($post_id)) {
               	 $catIds = array_filter([$mainCatId, $subCatId]); // Remove any false values
	                wp_set_post_terms($post_id, $catIds, 'category');
   	         }
   	      }
 			   if (!is_wp_error($featured_image_id)) {
  			  		set_post_thumbnail($post_id, $featured_image_id);
			   }

            if (!is_wp_error($post_id) && $post_id) {
            	 $focus_keyphrase = wp_strip_all_tags($content_item['title']);
            	 $site_name = get_bloginfo('name');
            	 if(strlen($focus_keyphrase) <= 45) 
			          $seo_title = $focus_keyphrase . ' - '.$site_name;
			         else 
			          $seo_title = $focus_keyphrase;
			        
                update_post_meta($post_id, 'saraID', $content_item['saraID']);
                update_post_meta($post_id, 'customerID', $content_item['customerID']);
                update_post_meta($post_id, 'orderID', $content_item['orderID']);
					 update_post_meta($post_id, '_yoast_wpseo_focuskw', $focus_keyphrase);
		          update_post_meta($post_id, '_yoast_wpseo_title', $seo_title);
        			 update_post_meta($post_id, '_yoast_wpseo_metadesc', $meta_description);
        			 update_post_meta($post_id, '_aioseo_title', $seo_title);
        			 update_post_meta($post_id, '_aioseo_description', $meta_description);
        			 
					 // Check for errors
					 if ($wpdb->last_error) {
					     error_log("aioseo failed db call: " . $wpdb->last_error);
					 }
                set_transient('seosara_admin_notice', ['message' => 'Post processed successfully. ', 'type' => 'success'], 3600);
    	     } else {
                set_transient('seosara_admin_notice', ['message' => 'Error processing post: ' . $post_id->get_error_message(), 'type' => 'error'], 3600);
           }
        }
        return;
    }
}


function seosara_wp_redirect_after_activation() {
    if (get_transient('seosara_activation_redirect')) {
        delete_transient('seosara_activation_redirect');
        // Check if the user can manage options (to prevent redirecting users who won't have access to the settings page)
        if (!current_user_can('manage_options')) {
            return;
        }
        // Perform the redirect to your settings page
        wp_redirect(admin_url('admin.php?page=seosara-api-key'));
        exit;
    }
}

function seosara_wp_order_article($keyword,$ctype) {
    // Sanitize the keyword to ensure safe use in the request
    $sanitized_keyword = sanitize_text_field($keyword);

    // Retrieve the site URL and API Key from WordPress options  
    $site_url = get_option('siteurl');
    $api_key = get_option('seosara_api_key');
    $email = get_option('seosara_email');
	 // Get the user by email address
	 $user = get_user_by('email', $email);
	 if ($user) {
	    // User found, $user->ID will give you the user's ID
	    $admin_id = $user->ID;
	    // You can now use $user_id for operations requiring the user's ID
	 } else { 	         
	 	// Get admin users
      $admin_users = get_users(['role' => 'administrator', 'number' => 1]);
      if (!empty($admin_users)) {
          // Assume the first admin user's ID as the author ID
         $admin_id = $admin_users[0]->ID;
  	   } else {
     	   // Fallback if no admin users are found, though unlikely
        	$admin_id = 1; // Typically the first user is an admin
      }
    }	
    // Prepare the data to be sent in the request
    $body = json_encode([
        'keyword_1' => $sanitized_keyword,
        'website' => $site_url,
        'email' => $email,
        'customer_id'    => $admin_id,
        'ctype'    => $ctype,
        'key' => $api_key,
    ]);

    // Set up the request arguments, including the HTTP Authorization header
    $args = [
        'method'      => 'POST',
        'body'        => $body,
        'headers'     => [
            'Content-Type'      => 'application/json',
            'Authorization'     => 'Bearer ' . $api_key,
        ],
        'timeout'     => 45,
        'redirection' => 5,
        'httpversion' => '1.0',
        'blocking'    => true,
        'sslverify'   => true,  // Set to false only in a non-production environment
    ];

    // Send the request to the specified endpoint
    $response = wp_remote_post('https://bot.seosara.ai/bot-GPT_que.php', $args);

// Check if the request returned an error
    if (is_wp_error($response)) {
        $error_message = $response->get_error_message();
        set_transient('seosara_admin_notice', ['message' => 'Error: ' . $error_message, 'type' => 'error'], 30);
    } else {
        // Decode the JSON response
        $response_body = wp_remote_retrieve_body($response);
        $decoded_response = json_decode($response_body, true);

        if (isset($decoded_response['status']) && $decoded_response['status'] === 'success') {
            // Handle success status
            if($decoded_response['credits'] != '')  update_option('seosara_credits', $decoded_response['credits']);
            set_transient('seosara_admin_notice', ['message' => $decoded_response['message'], 'type' => 'success'], 30);
        } elseif (isset($decoded_response['status']) && $decoded_response['status'] === 'error') {
            // Handle error status
            if($decoded_response['credits'] != '')  update_option('seosara_credits', $decoded_response['credits']);
            set_transient('seosara_admin_notice', ['message' => $decoded_response['message'], 'type' => 'error'], 30);
        } else {
        	$jn = json_encode($response_body);
            if($decoded_response['credits'] != '')  update_option('seosara_credits', $decoded_response['credits']);
            set_transient('seosara_admin_notice', ['message' => 'Unknown response from the API - '.$jn, 'type' => 'error'], 30);
        }
    }

    // Redirect back to the 'Order Content' page
    wp_redirect(admin_url('admin.php?page=seosara-order-content'));
    exit;
}

function seosara_wp_display_admin_notice() {
    if ($notice = get_transient('seosara_admin_notice')) {
        $class = 'notice';
        $class .= $notice['type'] === 'error' ? ' notice-error' : ' notice-success';
        echo '<div class="' . $class . ' is-dismissible"><p>' . esc_html($notice['message']) . '</p></div>';
        delete_transient('seosara_admin_notice');
    }
}

function seosara_wp_order_content_page() {
    // Fetch the current value of 'seosara_credits' from the database
    $credits = get_option('seosara_credits', '0'); // Default to '0' if not set
    $ptype = get_option('seosara_post_type'); 
	 $ptype = ucfirst($ptype);
    // Define the URL for your plugin's image
    $image_url = plugins_url('seosara.png', __FILE__); // Adjust the path as necessary
    
    if($credits < 5) $ctype = 'free';
    else $ctype = 'paid';

    ?>
    	<h2 style="text-align: center;">SEOSARA</h2>
   <div class="wrap" style="margin:0px auto;max-width:650px;text-align: center;">
        <img src="<?php echo esc_attr($image_url); ?>" alt="SEOSARA.AI Logo" style="margin-bottom: 20px;"/>
        <div style="text-align:left;margin-left:25px">
	        <h3>Welcome to SEOSARA Content Ordering!</h3>
   	     <h3>Credit Balance: <?php echo esc_html($credits); ?> - <a style="text-decoration:none;" target="_blank" href="https://seosara.ai/shop/" title="Buy Credits">Buy Credits</a></h3>
   	     <style>
   	     		#seosara_keyword {margin-right:15px;}
   	     </style>
			  <form method="post">
    			  <div class="input-wrapper" style="position: relative; margin-bottom: 20px;">
     					  	     <input type="text" id="seosara_keyword" name="seosara_keyword" required="required" placeholder="Enter keyword here" style="font-size:16px;"/><input type="submit" name="seosara_order_content" class="button button-primary" value="Order <?php echo $ptype; ?> Now"/>&nbsp; &nbsp; <a style="text-decoration:none;font-size:1.3em;font-weight:600;" href="/wp-admin/admin.php?page=seosara-api-key">Change Post Type</a>
                <p><strong>Upgraded or Free?</strong> - <a style="text-decoration:none;" target="_blank" href="https://seosara.ai/product/free-api-key/" title="Get a Free API Key">Get a Free API Key</a></p>
                <select name="seosara_type">
                    <option value="paid" <?php if($ctype == 'paid') echo 'selected'; ?>>Upgraded</option>
                    <option value="free" <?php if($ctype == 'free') echo 'selected'; ?>>Free</option>
                </select>
        			  <!-- Hidden hint -->
        			  <div class="hint" style="position: absolute; bottom: 100%; left: 0; background-color: #f9f9f9; padding: 5px; border: 1px solid #ddd; display: none; font-size: 14px;">Be sure to capitalize acronyms like AI and SEO!.</div>
    			  </div>
    			  
			  </form>
      	  <p></p>
      	  <p>You're just a few steps away from ordering high-quality, SEO-optimized content tailored to your specific needs. Here's how you can start enriching your website with engaging content:</p>
	        <ol>
   	     		<li><strong>Enter Your Keyword: </strong> Begin by inputting the keyword or topic you want to generate content for in the designated field. This helps us tailor the content specifically to your requirements.</li>
					<li><strong>Order Content: </strong> Once you've entered your keyword, click the "Order Content" button. Each piece of content we create for you will be approximately 2500 words in length, complete with a relevant Q&amp;A section and a resource section, ensuring comprehensive coverage of your topic.</li>
					<li><strong>Credit Balance &amp; Cost: </strong> Keep an eye on your credit balance displayed on this page. Each content order will cost 5 credits. If you need more credits, you can easily top up your balance to continue ordering without interruptions.</li>
					<li><strong>Completion Time: </strong> Your custom content will be ready in 15 to 30 minutes. We strive to deliver high-quality content efficiently, so you can quickly move forward with your content strategy.</li>
					<li><strong>Publishing Your Content: </strong> Once your content is ready, you'll find it under the "SEOSARA Pages" or "SEOSARA Posts" tab in the WP admin panel depending on the post type chosen. From there, you can review, edit if necessary, and publish the content directly to your site.</li>
   	     </ol>
      	  <p>Don't have enough credits or new to SEOSARA? No problem! Visit our <a target="_blank" href="https://seosara.ai/shop/">SEOSARA.AI</a> to purchase credits and learn more about how SEOSARA can transform your website's content strategy.</p>
	        <p>We're excited to see how our content will help you achieve your SEO and content goals. If you have any questions or need assistance, don't hesitate to reach out.</p>
      	  <p><strong>Your credit balance will update with every order.</strong></p>
   	     <p>Thank you for choosing SEOSARA. Let's create something amazing together!</p>
	     </div>
    </div>
        <style>
            .input-wrapper {
                position: relative;
                margin-bottom: 20px;
            }

            .hint {
                position: absolute;
                bottom: 100%; /* Position above the input */
                left: 0;
                background-color: #f9f9f9;
                padding: 5px;
                border: 1px solid #ddd;
                display: none; /* Initially hidden */
                font-size: 16px;
                margin-bottom: 10px; /* Space between hint and input */
            }
        </style>
        <script>
            document.addEventListener('DOMContentLoaded', function() {
                var keywordInput = document.getElementById('seosara_keyword');

                keywordInput.addEventListener('focus', function() {
                    this.parentNode.querySelector('.hint').style.display = 'block';
                });

                keywordInput.addEventListener('blur', function() {
                    this.parentNode.querySelector('.hint').style.display = 'none';
                });
            });
        </script>
    <?php

    // Check if the form has been submitted
    if (isset($_POST['seosara_order_content']) && !empty($_POST['seosara_keyword']) && !empty($_POST['seosara_type'])) {
        $keyword = sanitize_text_field($_POST['seosara_keyword']);
        $ctype = sanitize_text_field($_POST['seosara_type']);
        seosara_wp_order_article($keyword,$ctype);
    }
}

function seosara_wp_activate_plugin() {
    // Set default options or perform other setup tasks
    add_option('seosara_api_key', '');
    add_option('seosara_credits', '0');
    add_option('seosara_post_type', 'post');
    add_option('seosara_post_status', 'draft');
    $email = get_option('admin_email');
    add_option('seosara_email', $email);
    
    // Other activation tasks like creating custom database tables
	  set_transient('seosara_activation_redirect', 1, 30);   
}

function seosara_wp_deactivate_plugin() {
    // Remove the API key from the database upon plugin deactivation
    delete_option('seosara_api_key');
    delete_option('seosara_credits');
    delete_option('seosara_post_type');
    delete_option('seosara_post_status');
    delete_option('seosara_email');
    
}


function seosara_wp_filter_pages_by_meta($query) {
    // Check if we're in the admin area, on the main query, and on the Pages list
    if (is_admin() && $query->is_main_query() && isset($_GET['post_type']) && ($_GET['post_type'] == 'page' || $_GET['post_type'] == 'post')) {
         if (!isset($_GET['saraID_filter'])) return;

        // Modify the query to filter by the 'saraID' meta key
        $meta_query = array(
            array(
                'key'     => 'saraID',
                'compare' => 'EXISTS',
            ),
        );
        $query->set('meta_query', $meta_query);
    }
}

// seosara_settings_page() displays the page content for the settings submenu
function seosara_wp_api_key_page() {
    $image_url = plugins_url('seosara.png', __FILE__); // Ensure the image name is correct
    ?>
    <h2 style="text-align: center;">SEOSARA</h2>
    <div class="wrap" style="margin:0px auto;max-width:650px;text-align: center;">
        <img src="<?php echo $image_url; ?>" alt="SEOSARA.AI Logo" style="margin-bottom: 20px;"/> 
        <div style="text-align:left;margin-left:20px">
            <h3>Welcome to SEOSARA! The Content Generator. &nbsp; &nbsp;  <a style="text-decoration:none;" href="/wp-admin/admin.php?page=seosara-order-content">Order Content</a></h3>
            <h3><a target="_blank" style="text-decoration:none;" href="https://seosara.ai/shop/">Get an API Key</a></h3>
            <p>Congratulations on taking the first step towards elevating your WordPress site with AI-powered, SEO-optimized content! To unlock the full potential of this plugin and start generating top-notch content, please enter your API key and preferred post type for the articles to be generated in the fields below.</p>
            <p>Our goal is to make your experience as seamless as possible. If you have any questions or need assistance, we're here to help.</p>
            <p>Thank you for choosing our plugin to enhance your website's content strategy. Let's make your site more engaging and SEO-friendly together!</p>
            <p>Don't have an API key yet? No worries! Visit <a target="_blank" href="https://seosara.ai/shop/">seosara.ai</a> to get your key and embark on your journey to effortless content creation.</p>
            <form method="post" action="options.php">
                <?php settings_fields('seosara-settings-group'); ?>
                <?php do_settings_sections('seosara-settings-group'); ?>
                <input type="text" name="seosara_api_key" value="<?php echo esc_attr(get_option('seosara_api_key')); ?>" placeholder="Enter your API key" />
                <p><strong>Select the Post Type for Generated Content:</strong></p>
                <select name="seosara_post_type">
                    <option value="page" <?php selected(get_option('seosara_post_type'), 'page'); ?>>Page</option>
                    <option value="post" <?php selected(get_option('seosara_post_type'), 'post'); ?>>Post</option>
                </select>
                <p><strong>If you would like to review the content before it is published, be sure to set the status below to draft:</strong></p>
                <select name="seosara_post_status">
                    <option value="publish" <?php selected(get_option('seosara_post_status'), 'publish'); ?>>Publish</option>
                    <option value="draft" <?php selected(get_option('seosara_post_status'), 'draft'); ?>>Draft</option>
                </select>
                <p><strong>Enter the Email Address Associated With the Above Key:</strong></p>
	                 <input type="email" name="seosara_email" value="<?php echo esc_attr(get_option('seosara_email')); ?>" placeholder="Enter your email" />
                <?php submit_button(); ?>
            </form>
        </div>      
    </div>
    <?php
}

function seosara_wp_admin_init(){
    register_setting('seosara-settings-group', 'seosara_api_key', 'seosara_wp_sanitize_api_key');
    register_setting('seosara-settings-group', 'seosara_post_type', 'seosara_sanitize_post_type');
    register_setting('seosara-settings-group', 'seosara_post_status', 'seosara_sanitize_post_status');
    // Register the Email setting
    register_setting('seosara-settings-group', 'seosara_email', array(
        'type' => 'string',
        'sanitize_callback' => 'sanitize_email',
        'default' => '',
    ));
}

function seosara_wp_sanitize_api_key($input) {
    // Sanitize $input as required
    return $input; // Return sanitized input
}

function seosara_wp_after_api_key_save($old_value, $value) {
    if ($value !== $old_value) {
		set_transient('seosara_admin_notice', ['message' => 'Your Settings have been saved.', 'type' => 'success'], 30);        // Your custom function logic here
    }
}

function seosara_wp_ensure_category_exists($catName) {
    if (empty($catName)) return false;

    $term = get_term_by('name', $catName, 'category');
    if ($term) {
        // Category exists
        return $term->term_id;
    } else {
        // Category doesn't exist, so create it
        $new_term = wp_insert_term($catName, 'category');
        if (!is_wp_error($new_term)) {
            return $new_term['term_id'];
        }
    }

    return false;
}

function seosara_wp_add_action_links($links) {
    // Define the settings link.
    $settings_link = '<a target="_blank" href="' . esc_url('https://seosara.ai/ai-content-writer-wp/') . '">Developer Website</a>';

    // Check if the $links is an array to avoid any unexpected errors.
    if (is_array($links)) {
        // Add the settings link to the beginning of the links array.
        array_unshift($links, $settings_link);
    }

    return $links;
}