document.querySelectorAll('.SEOSARAbox').forEach(element => {
    element.addEventListener('click', function(event) {
        event.preventDefault(); // Prevent the default link behavior

        const href = this.getAttribute('href'); // Get the image source from the href attribute
        const alt = this.children[0].getAttribute('alt'); // Assuming the image is the first child, get the alt attribute

        // Create an Image object to load the image and determine its aspect ratio
        const img = new Image();
        img.src = href;
        img.onload = function() {
            const aspectRatio = this.naturalWidth / this.naturalHeight;
            let width = '95vw', height = '95vh'; // Default for square images

            if (aspectRatio > 1) {
                // Landscape image
                width = '95vw';
                height = 'auto';
            } else if (aspectRatio < 1) {
                // Portrait image
                width = 'auto';
                height = '95vh';
            }

            // Create the popup container and its content with the determined dimensions
            const popup = document.createElement('div');
            popup.setAttribute('id', 'imagePopup');
            popup.innerHTML = `
                <div id="popupContent" style="position: fixed; top: 50%; left: 50%; transform: translate(-50%, -50%); background-color: #fff; padding: 0px; box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1); z-index: 1000;">
                    <img src="${href}" alt="${alt}" style="max-width: ${width}; max-height: ${height}; display: block; margin: 0 auto;">
                    <div id="caption" style="color: #fff; background-color: #000; text-align: center; padding: 10px 0;">${alt}</div>
                </div>
            `;

            // Append the popup to the body
            document.body.appendChild(popup);

            // Add an overlay for aesthetics and closing functionality
            const overlay = document.createElement('div');
            overlay.setAttribute('id', 'overlay');
            overlay.style.cssText = 'position: fixed; top: 0; left: 0; width: 100%; height: 100%; background-color: rgba(0, 0, 0, 0.5); z-index: 999;';
            document.body.appendChild(overlay);

            // Close the popup when the overlay is clicked
            overlay.addEventListener('click', function() {
                popup.remove();
                this.remove(); // 'this' refers to the overlay here
            });
        };
    });
});


document.querySelectorAll('.justify-content-center a').forEach(link => {
    link.addEventListener('click', function(event) {
        event.preventDefault(); // Prevent the default jump to section behavior

        const targetId = this.getAttribute('href');
        const targetElement = document.querySelector(targetId);

        if (targetElement) {
            smoothScrollTo(targetElement.offsetTop, 800); // 1000ms duration for the scroll
        }
    });
});

function smoothScrollTo(targetPosition, duration) {
    const startPosition = window.pageYOffset;
    const distance = targetPosition - startPosition;
    let startTime = null;

    function animation(currentTime) {
        if (startTime === null) startTime = currentTime;
        const timeElapsed = currentTime - startTime;
        const run = ease(timeElapsed, startPosition, distance, duration);

        window.scrollTo(0, run);

        if (timeElapsed < duration) requestAnimationFrame(animation);
    }

    function ease(t, b, c, d) {
        t /= d / 2;
        if (t < 1) return c / 2 * t * t + b;
        t--;
        return -c / 2 * (t * (t - 2) - 1) + b;
    }

    requestAnimationFrame(animation);
}